@@ -51,7 +51,7 @@ protected:
   virtual void init_menus_help(); //override
   virtual void init_toolbars(); //override
   virtual void init_create_document(); //override
-  virtual void on_document_load(); //override.
+  virtual bool on_document_load(); //override.
 
   virtual bool offer_new_or_existing();
 
@@ -63,6 +63,8 @@ protected:
 
   virtual Bakery::App* new_instance(); //Override
 
+  virtual bool recreate_database(); //return indicates success.
+  
   typedef Bakery::App_WithDoc_Gtk type_base;
 
   //Widgets:
