@@ -39,6 +39,8 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
 {
   m_layout_name = "details";
 
+  add_view(&m_FlowTable); //Allow this to access the document too.
+
   Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "window_data_layout_details"); //TODO: Use a generic layout dialog?
   if(refXml)
   {
@@ -63,6 +65,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Paned.add(m_FlowTable);
 
   //Related records:
+  /*
   m_Label_Related.set_text(gettext("<b>Related Records</b>"));
   m_Label_Related.set_use_markup(true);
   m_Frame_Related.set_label_widget(m_Label_Related);
@@ -73,6 +76,7 @@ Box_Data_Details::Box_Data_Details(bool bWithNavButtons /* = true */)
   m_Notebook_Related.set_border_width(6);
   m_Alignment_Related.add(m_Notebook_Related);
   m_Paned.add(m_Frame_Related);
+  */
 
   //Add or delete record:
   m_HBox.pack_start(m_Button_New, Gtk::PACK_SHRINK);
@@ -114,21 +118,21 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value() const
   return m_primary_key_value;
 }
 
-void Box_Data_Details::init_db_details(const Glib::ustring& strDatabaseName, const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::init_db_details(const Glib::ustring& strTableName, const Gnome::Gda::Value& primary_key_value)
 {
   m_primary_key_value = primary_key_value;
 
-  Box_DB_Table::init_db_details(strDatabaseName, strTableName);
+  Box_DB_Table::init_db_details(strTableName);
 }
 
-void Box_Data_Details::init_db_details(const Gnome::Gda::Value& primary_key_value)
+void Box_Data_Details::refresh_db_details(const Gnome::Gda::Value& primary_key_value)
 {
-  init_db_details(get_database_name(), get_table_name(), primary_key_value);
+  init_db_details(get_table_name(), primary_key_value);
 }
 
-void Box_Data_Details::init_db_details_blank()
+void Box_Data_Details::refresh_db_details_blank()
 {
-  init_db_details( Gnome::Gda::Value() );
+  refresh_db_details( Gnome::Gda::Value() );
 }
 
 void Box_Data_Details::fill_from_database_layout()
@@ -141,6 +145,8 @@ void Box_Data_Details::fill_from_database_layout()
   Document_Glom* document = dynamic_cast<Document_Glom*>(get_document());
   if(document)
   {
+    m_FlowTable.set_table(m_strTableName); //This allows portals to get full Relationship information
+    
     //This map of layout groups will also contain the field information from the database:
     Document_Glom::type_mapLayoutGroupSequence layout_groups = get_data_layout_groups("details");
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = layout_groups.begin(); iter != layout_groups.end(); ++iter)
@@ -155,23 +161,26 @@ void Box_Data_Details::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
 
-  Box_DB_Table::fill_from_database();
-         
-  type_vecFields listFieldsToShow = get_fields_to_show();
-  m_Fields =  listFieldsToShow;
-  if(!listFieldsToShow.empty())
+  try
   {
-    fill_from_database_layout(); //TODO: Only do this when the layout has changed.
+    //TODO: This should keep the connection open, so we don't need to 
+    //reconnect many times..
+    sharedptr<SharedConnection> sharedconnection = connect_to_server();
 
-    Field field_primary_key;
-    bool primary_key_found  = get_field_primary_key(field_primary_key);
 
-    if(primary_key_found && !GlomConversions::value_is_empty(m_primary_key_value)) //If there is a record to show:
+    Box_DB_Table::fill_from_database();
+         
+    type_vecFields listFieldsToShow = get_fields_to_show();
+    m_Fields =  listFieldsToShow;
+    if(!listFieldsToShow.empty())
     {
-      try
-      {
-        sharedptr<SharedConnection> sharedconnection = connect_to_server();
+      fill_from_database_layout(); //TODO: Only do this when the layout has changed.
 
+      Field field_primary_key;
+      bool primary_key_found  = get_field_primary_key(field_primary_key);
+
+      if(primary_key_found && !GlomConversions::value_is_empty(m_primary_key_value)) //If there is a record to show:
+      {
         if(sharedconnection)
         {
           Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
@@ -211,23 +220,25 @@ void Box_Data_Details::fill_from_database()
           }
         }
       }
-      catch(std::exception& ex)
+      else
       {
-        handle_error(ex);
+        //Show blank record:
+        //TODO
       }
-    }
-    else
-    {
-      //Show blank record:
-      //TODO
-    }
-  } //if(!listFieldsToShow.empty())
+    } //if(!listFieldsToShow.empty())
 
-  fill_related();
+    //fill_related();
 
-  fill_end();
+    fill_end();
+
+  }
+  catch(std::exception& ex)
+  {
+    handle_error(ex);
+  }
 }
 
+/*
 void Box_Data_Details::fill_related()
 {
   if(!m_bDoNotRefreshRelated)
@@ -292,7 +303,7 @@ void Box_Data_Details::fill_related()
            bool test = get_fields_for_table_one_field(m_strTableName, from_field, field);
            if(test)
            {
-             pBox->init_db_details(get_database_name(), relationship, value, m_primary_key_value);
+             pBox->init_db_details(relationship, value, m_primary_key_value);
              pBox->show_all();
 
 
@@ -312,6 +323,7 @@ void Box_Data_Details::fill_related()
   }
 
 }
+*/
 
 void Box_Data_Details::on_button_new()
 {
@@ -333,14 +345,14 @@ void Box_Data_Details::on_button_new()
         if(parsed)
         {
           record_new(false /* use entered field data */, primary_key_value);
-          init_db_details(primary_key_value);
+          refresh_db_details(primary_key_value);
         }
       }
       else
       {
         //It's not an auto-increment primary key,
         //so just blank the fields ready for a primary key later.
-        init_db_details_blank(); //shows blank record.
+        refresh_db_details_blank(); //shows blank record.
       }
     }
 
@@ -416,10 +428,12 @@ Gnome::Gda::Value Box_Data_Details::get_primary_key_value_selected()
   return Gnome::Gda::Value(strResult);
 }
 
+/*
 void Box_Data_Details::on_related_user_requested_details(Gnome::Gda::Value key_value, Glib::ustring strTableName)
 {
   signal_user_requested_related_details().emit(strTableName, key_value);
 }
+*/
 
 void Box_Data_Details::on_related_record_added(Gnome::Gda::Value /* strKeyValue */, Glib::ustring /* strFromKeyName */)
 {
@@ -474,10 +488,12 @@ Box_Data_Details::type_signal_record_deleted Box_Data_Details::signal_record_del
   return m_signal_record_deleted; 
 }
 
+/*
 Box_Data_Details::type_signal_user_requested_related_details Box_Data_Details::signal_user_requested_related_details()
 {
   return m_signal_user_requested_related_details;
 }
+*/
 
 void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const Gnome::Gda::Value& field_value)
 {
@@ -524,6 +540,7 @@ void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const
            do_lookups(field, field_value, fieldInfoPK, primary_key_value);
 
            //If this is a foreign key then refresh the related records:
+           /*
            bool bIsForeignKey = false;
            Document_Glom::type_vecRelationships vecRelationships = m_pDocument->get_relationships(m_strTableName);
            for(Document_Glom::type_vecRelationships::iterator iter = vecRelationships.begin(); iter != vecRelationships.end(); iter++)
@@ -539,6 +556,7 @@ void Box_Data_Details::on_flowtable_field_edited(const Glib::ustring& id, const
 
            if(bIsForeignKey)
              fill_related();
+           */
          }
 
        }
