@@ -59,7 +59,8 @@ Box_Data_List::Box_Data_List()
 
   //Groups are not very helpful for a list view:
   //m_pDialogLayout->set_show_groups(false);
- 
+
+  m_AddDel.show();
 }
 
 Box_Data_List::~Box_Data_List()
@@ -70,16 +71,17 @@ void Box_Data_List::fill_from_database()
 {
   Bakery::BusyCursor(*get_app_window());
  
-  Box_DB_Table::fill_from_database();
-
-  m_AddDel.remove_all();
-
-  //Field Names:
-  fill_column_titles();
-   
   try
   {
     sharedptr<SharedConnection> sharedconnection = connect_to_server();
+
+    Box_DB_Table::fill_from_database();
+
+    m_AddDel.remove_all();
+
+    //Field Names:
+    fill_column_titles();
+   
     if(sharedconnection)
     {    
       Glib::RefPtr<Gnome::Gda::Connection> connection = sharedconnection->get_gda_connection();
@@ -144,12 +146,17 @@ void Box_Data_List::fill_from_database()
                   Gnome::Gda::Value value = result->get_value_at(uiCol, result_row);
                   guint index = 0;
 
+                  //g_warning("list fill: field_name=%s", iterFields->get_name().c_str());
+                  //g_warning("  value_as_string=%s", value.to_string().c_str());
+
                   //TODO_Performance: This searches m_Fields again each time:
                   bool test = get_field_column_index(iterFields->get_name(), index);
                   ++iterFields;
 
                   if(test)
                     m_AddDel.set_value(tree_iter, index, value);
+		  else
+		    g_warning("  get_field_column_index failed.");
                 }
               }
             }
@@ -159,18 +166,19 @@ void Box_Data_List::fill_from_database()
         }
       } //If !m_Fields.empty()
     }
+   
+    //Select first record:
+    Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
+    if(refModel)
+      m_AddDel.select_item(refModel->children().begin());
+ 
+    fill_end();
+
   }
   catch(std::exception& ex)
   {
     handle_error(ex);
   }
-   
-  //Select first record:
-  Glib::RefPtr<Gtk::TreeModel> refModel = m_AddDel.get_model();
-  if(refModel)
-    m_AddDel.select_item(refModel->children().begin());
- 
-  fill_end();
 }
 
 void Box_Data_List::on_adddel_user_requested_add()
@@ -318,6 +326,8 @@ void Box_Data_List::on_adddel_user_reordered_columns()
 
 void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row, guint col)
 {
+  g_warning("on_adddel_user_changed():");
+  
   const Gnome::Gda::Value primary_key_value = get_primary_key_value(row);
   if(!GlomConversions::value_is_empty(primary_key_value)) //If the record's primary key is filled in:
   {
@@ -346,6 +356,14 @@ void Box_Data_List::on_adddel_user_changed(const Gtk::TreeModel::iterator& row,
           //Get-and-set values for lookup fields, if this field triggers those relationships:
           do_lookups(row, field, field_value, field_primary_key, primary_key_value);
         }
+        else
+        {
+          g_warning("Box_Data_List::on_adddel_user_changed(): get_fields_for_table_one_field() failed: m_strTableName=%s, field_name=%s", m_strTableName.c_str(), field_name.c_str());
+        }
+      }
+      else
+      {
+        g_warning("Box_Data_List::on_adddel_user_changed(): primary key not found");
       }
     }
     catch(const std::exception& ex)
