@@ -39,7 +39,9 @@ public:
    */
   virtual void set_document(Document_Glom* document, const Glib::ustring& table_name);
 
-  virtual bool get_field_chosen(Field& field) const;
+  void select_item(const Field& field);
+    
+  bool get_field_chosen(Field& field) const;
   
 protected:
 
