@@ -20,6 +20,7 @@
 
 #include "dialog_layout_details.h"
 #include "dialog_choose_field.h"
+#include "dialog_choose_relationship.h"
 //#include <libgnome/gnome-i18n.h>
 #include <libintl.h>
 #include <sstream> //For stringstream
@@ -30,6 +31,8 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   m_button_field_up(0),
   m_button_field_down(0),
   m_button_field_add(0),
+  m_button_field_add_group(0),
+  m_button_add_related(0),
   m_button_field_delete(0),
   m_button_field_edit(0)
 {
@@ -40,8 +43,8 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
     m_treeview_fields->enable_model_drag_source();
     m_treeview_fields->enable_model_drag_dest();
     
-    m_model_fields = TreeStore_Layout::create();
-    m_treeview_fields->set_model(m_model_fields);
+    m_model_items = TreeStore_Layout::create();
+    m_treeview_fields->set_model(m_model_items);
 
     // Append the View columns:
     // Use set_cell_data_func() to give more control over the cell attributes depending on the row:
@@ -57,22 +60,22 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
     //Connect to its signal:
     renderer_name->property_editable() = true;
     renderer_name->signal_edited().connect(
-      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_text), m_model_fields->m_columns.m_col_name) );
+      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_text), m_model_items->m_columns.m_col_name) );
+
 
-      
     //Title column:
     Gtk::TreeView::Column* column_title = Gtk::manage( new Gtk::TreeView::Column(gettext("Title")) );
     m_treeview_fields->append_column(*column_title);
-    
+
     Gtk::CellRendererText* renderer_title = Gtk::manage(new Gtk::CellRendererText);
     column_title->pack_start(*renderer_title);
     column_title->set_cell_data_func(*renderer_title, sigc::mem_fun(*this, &Dialog_Layout_Details::on_cell_data_title));
 
     //Connect to its signal:
     renderer_title->signal_edited().connect(
-      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_text), m_model_fields->m_columns.m_col_title) );
+      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_text), m_model_items->m_columns.m_col_title) );
+
 
-              
     //Columns-count column:
     Gtk::TreeView::Column* column_count = Gtk::manage( new Gtk::TreeView::Column(gettext("Columns Count")) );
     m_treeview_fields->append_column(*column_count);
@@ -83,7 +86,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
 
     //Connect to its signal:
     renderer_count->signal_edited().connect(
-      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_numeric), m_model_fields->m_columns.m_col_columns_count) );
+      sigc::bind( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_cell_edited_numeric), m_model_items->m_columns.m_col_columns_count) );
 
     //Respond to changes of selection:
     Glib::RefPtr<Gtk::TreeView::Selection> refSelection = m_treeview_fields->get_selection();
@@ -92,7 +95,7 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
       refSelection->signal_changed().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treeview_fields_selection_changed) );
     }
 
-    m_model_fields->signal_row_changed().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treemodel_row_changed) );
+    m_model_items->signal_row_changed().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_treemodel_row_changed) );
   }
 
   refGlade->get_widget("button_field_up", m_button_field_up);
@@ -110,7 +113,10 @@ Dialog_Layout_Details::Dialog_Layout_Details(BaseObjectType* cobject, const Glib
   refGlade->get_widget("button_field_add_group", m_button_field_add_group);
   m_button_field_add_group->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_add_group) );
 
-  refGlade->get_widget("button_field_edit", m_button_field_edit);
+  refGlade->get_widget("button_add_related", m_button_add_related);
+  m_button_add_related->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_add_related) );
+
+  refGlade->get_widget("button_edit", m_button_field_edit);
   m_button_field_edit->signal_clicked().connect( sigc::mem_fun(*this, &Dialog_Layout_Details::on_button_field_edit) );
 
   show_all_children();
@@ -126,53 +132,61 @@ void Dialog_Layout_Details::fill_group(const Gtk::TreeModel::iterator& iter, Lay
   {
     Gtk::TreeModel::Row row = *iter;
 
-    group.set_name( row[m_model_fields->m_columns.m_col_name] );
-    group.m_columns_count = row[m_model_fields->m_columns.m_col_columns_count];
-    group.m_title = row[m_model_fields->m_columns.m_col_title];
+    group.set_name( row[m_model_items->m_columns.m_col_name] );
+    group.m_columns_count = row[m_model_items->m_columns.m_col_columns_count];
+    group.m_title = row[m_model_items->m_columns.m_col_title];
 
     //Get child layout items:
     for(Gtk::TreeModel::iterator iterChild = row.children().begin(); iterChild != row.children().end(); ++iterChild)
     {
       Gtk::TreeModel::Row rowChild = *iterChild;
-      if(rowChild[m_model_fields->m_columns.m_col_is_group])
+      if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
       {
         //Recurse:
         LayoutGroup group_child;
         fill_group(iterChild, group_child);
         group.add_item(group_child);
       }
-      else
+      else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_PORTAL)
+      {
+        LayoutItem_Portal portal;
+        portal.set_relationship(  rowChild[m_model_items->m_columns.m_col_relationship] );
+        group.add_item(portal);
+      }
+      else if(rowChild[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_FIELD)
       {
         //Add field:
         LayoutItem_Field field;
-        field.set_name( rowChild[m_model_fields->m_columns.m_col_name] );
-        
+        field.set_name( rowChild[m_model_items->m_columns.m_col_name] );
+
         group.add_item(field);
       }
     }
   }
 }
 
+
+
 void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, const LayoutGroup& group)
 {
   Gtk::TreeModel::iterator iterNewGroup;
   if(!parent)
   {
     //Add it at the top-level, because nothing was selected:
-    iterNewGroup = m_model_fields->append();
+    iterNewGroup = m_model_items->append();
   }
   else
   {
-    iterNewGroup = m_model_fields->append(parent->children());
+    iterNewGroup = m_model_items->append(parent->children());
   }
 
   if(iterNewGroup)
   {
     Gtk::TreeModel::Row row = *iterNewGroup;
-    row[m_model_fields->m_columns.m_col_is_group] = true;
-    row[m_model_fields->m_columns.m_col_name] = group.get_name();
-    row[m_model_fields->m_columns.m_col_columns_count] = group.m_columns_count;
-    row[m_model_fields->m_columns.m_col_title] = group.m_title;
+    row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_GROUP;
+    row[m_model_items->m_columns.m_col_name] = group.get_name();
+    row[m_model_items->m_columns.m_col_columns_count] = group.m_columns_count;
+    row[m_model_items->m_columns.m_col_title] = group.m_title;
 
     //Add the child items:
     LayoutGroup::type_map_const_items items = group.get_items();
@@ -184,14 +198,25 @@ void Dialog_Layout_Details::add_group(const Gtk::TreeModel::iterator& parent, co
         add_group(iterNewGroup, *child_group); //recursive
       else
       {
-        const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
-        if(field) //If it is a field
+        const LayoutItem_Portal* portal = dynamic_cast<const LayoutItem_Portal*>(item);
+        if(portal) //If it is a portal
         {
-          //Add the field to the treeview:
-          Gtk::TreeModel::iterator iterField = m_model_fields->append(iterNewGroup->children());
+          Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
           Gtk::TreeModel::Row row = *iterField;
-          row[m_model_fields->m_columns.m_col_is_group] = false;
-          row[m_model_fields->m_columns.m_col_name] = field->get_name();
+          row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
+          row[m_model_items->m_columns.m_col_relationship] = portal->get_relationship();
+        }
+        else
+        {
+          const LayoutItem_Field* field = dynamic_cast<const LayoutItem_Field*>(item);
+          if(field) //If it is a field
+          {
+            //Add the field to the treeview:
+            Gtk::TreeModel::iterator iterField = m_model_items->append(iterNewGroup->children());
+            Gtk::TreeModel::Row row = *iterField;
+            row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
+            row[m_model_items->m_columns.m_col_name] = field->get_name();
+          }
         }
       }
     }
@@ -242,7 +267,7 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
     //Show the field layout
     typedef std::list< Glib::ustring > type_listStrings;
 
-    m_model_fields->clear();
+    m_model_items->clear();
 
     //guint field_sequence = 1; //0 means no sequence
     //guint group_sequence = 1; //0 means no sequence
@@ -253,12 +278,12 @@ void Dialog_Layout_Details::set_document(const Glib::ustring& layout, Document_G
       add_group(Gtk::TreeModel::iterator() /* null == top-level */, group);
     }
 
-    //treeview_fill_sequences(m_model_fields, m_model_fields->m_columns.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
+    //treeview_fill_sequences(m_model_items, m_model_items->m_columns.m_col_sequence); //The document should have checked this already, but it does not hurt to check again.
   }
 
   //Open all the groups:
   m_treeview_fields->expand_all();
-  
+
   m_modified = false;
 }
 
@@ -273,7 +298,7 @@ void Dialog_Layout_Details::enable_buttons()
     {
       //Disable Up if It can't go any higher.
       bool enable_up = true;
-      if(iter == m_model_fields->children().begin())
+      if(iter == m_model_items->children().begin())
         enable_up = false;  //It can't go any higher.
 
       m_button_field_up->set_sensitive(enable_up);
@@ -284,7 +309,7 @@ void Dialog_Layout_Details::enable_buttons()
       iterNext++;
 
       bool enable_down = true;
-      if(iterNext == m_model_fields->children().end())
+      if(iterNext == m_model_items->children().end())
         enable_down = false;
 
       m_button_field_down->set_sensitive(enable_down);
@@ -312,7 +337,7 @@ void Dialog_Layout_Details::on_button_field_delete()
     Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
     if(iter)
     {
-      m_model_fields->erase(iter);
+      m_model_items->erase(iter);
 
       m_modified = true;
     }
@@ -332,14 +357,14 @@ void Dialog_Layout_Details::on_button_field_up()
       if(parent)
         is_first = (iter == parent->children().begin());
       else
-        is_first = (iter == m_model_fields->children().begin());
-        
+        is_first = (iter == m_model_items->children().begin());
+
       if(!is_first)
       {
         Gtk::TreeModel::iterator iterBefore = iter;
         --iterBefore;
 
-        m_model_fields->iter_swap(iter, iterBefore);
+        m_model_items->iter_swap(iter, iterBefore);
 
         m_modified = true;
       }
@@ -366,12 +391,12 @@ void Dialog_Layout_Details::on_button_field_down()
       if(parent)
         is_last = (iterNext == parent->children().end());
       else
-        is_last = (iterNext == m_model_fields->children().end());
+        is_last = (iterNext == m_model_items->children().end());
 
       if(!is_last)
       {
         //Swap the sequence values, so that the one before will be after:
-        m_model_fields->iter_swap(iter, iterNext);
+        m_model_items->iter_swap(iter, iterNext);
 
         m_modified = true;
       }
@@ -384,25 +409,82 @@ void Dialog_Layout_Details::on_button_field_down()
 
 void Dialog_Layout_Details::on_button_field_add()
 {
-  Gtk::TreeModel::iterator parent;
+  Gtk::TreeModel::iterator parent = get_selected_group_parent();
 
-  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
-  if(refTreeSelection)
+  Field field;
+  bool test = offer_field_list(field);
+  if(test)
   {
-    Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
+    //Add the field details to the layout treeview:
+    Gtk::TreeModel::iterator iter;
+    if(parent)
+      iter = m_model_items->append(parent->children());
+    else
+    {
+      //Find the first group, and make the new row a child of that:
+      Gtk::TreeModel::iterator iter_first = m_model_items->children().begin();
+      if(iter_first)
+      {
+        Gtk::TreeModel::Row row = *iter_first;
+        if(row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
+          iter = m_model_items->append(iter_first->children());
+      }
+    }
+
     if(iter)
     {
       Gtk::TreeModel::Row row = *iter;
-      if(row[m_model_fields->m_columns.m_col_is_group])
-        parent = iter; //Add the item to this group.
-      else
-      {
-        //Add the item to the selected fields's parent group:
-        parent = iter->parent();
-      }
+      row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_FIELD;
+      row[m_model_items->m_columns.m_col_name] = field.get_name();
+      //row[m_model_items->m_columns.m_col_title] = field.get_title();
+
+      //Scroll to, and select, the new row:
+      Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+      if(refTreeSelection)
+        refTreeSelection->select(iter);
+
+      m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
     }
   }
-        
+}
+
+
+bool Dialog_Layout_Details::offer_relationship_list(Relationship& relationship)
+{
+  bool result = false;
+  
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_relationship");
+
+    Dialog_ChooseRelationship* dialog = 0;
+    refXml->get_widget_derived("dialog_choose_relationship", dialog);
+
+    if(dialog)
+    {
+      dialog->set_document(m_document, m_table_name);
+      int response = dialog->run();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen relationship:
+        result = dialog->get_relationship_chosen(relationship);
+      }
+
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
+
+bool Dialog_Layout_Details::offer_field_list(Field& field)
+{
+  bool result = false;
+  
   try
   {
     Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_choose_field");
@@ -417,52 +499,65 @@ void Dialog_Layout_Details::on_button_field_add()
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen field:
-        Field field;
-        dialog->get_field_chosen(field);
-
-        //Add the field details to the layout treeview:
-        Gtk::TreeModel::iterator iter;
-        if(parent)
-          iter = m_model_fields->append(parent->children());
-        else
-        {
-          //Find the first group, and make the new row a child of that:
-          Gtk::TreeModel::iterator iter_first = m_model_fields->children().begin();
-          if(iter_first)
-          {
-            Gtk::TreeModel::Row row = *iter_first;
-            if(row[m_model_fields->m_columns.m_col_is_group])
-              iter = m_model_fields->append(iter_first->children());
-          }
-        }
-
-        if(iter)
-        {
-          Gtk::TreeModel::Row row = *iter;
-          row[m_model_fields->m_columns.m_col_is_group] = false;
-          row[m_model_fields->m_columns.m_col_name] = field.get_name();
-          //row[m_model_fields->m_columns.m_col_title] = field.get_title();
-
-          //Scroll to, and select, the new row:
-          if(refTreeSelection)
-            refTreeSelection->select(iter);
-    
-          m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
-        }
+        result = dialog->get_field_chosen(field);
       }
-    }
 
-    delete dialog;      
+      delete dialog;
+    }
   }
   catch(const Gnome::Glade::XmlError& ex)
   {
     std::cerr << ex.what() << std::endl;
   }
 
+  return result;
 }
 
-void Dialog_Layout_Details::on_button_field_add_group()
+void Dialog_Layout_Details::on_button_add_related()
 {
+  Gtk::TreeModel::iterator parent = get_selected_group_parent();
+
+  Relationship relationship;
+  bool test = offer_relationship_list(relationship);
+  if(test)
+  {
+    //Add the field details to the layout treeview:
+    Gtk::TreeModel::iterator iter;
+    if(parent)
+      iter = m_model_items->append(parent->children());
+    else
+    {
+      //Find the first group, and make the new row a child of that:
+      Gtk::TreeModel::iterator iter_first = m_model_items->children().begin();
+      if(iter_first)
+      {
+        Gtk::TreeModel::Row row = *iter_first;
+        if(row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
+          iter = m_model_items->append(iter_first->children());
+      }
+    }
+
+    if(iter)
+    {
+      Gtk::TreeModel::Row row = *iter;
+      row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_PORTAL;
+      row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+      //row[m_model_items->m_columns.m_col_title] = field.get_title();
+
+      //Scroll to, and select, the new row:
+      Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+      if(refTreeSelection)
+        refTreeSelection->select(iter);
+
+      m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iter) );
+    }
+  }
+}
+
+Gtk::TreeModel::iterator Dialog_Layout_Details::get_selected_group_parent() const
+{
+  //Get the selected group, or a suitable parent group, or the first group:
+
   Gtk::TreeModel::iterator parent;
 
   Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
@@ -472,7 +567,7 @@ void Dialog_Layout_Details::on_button_field_add_group()
     if(iter)
     {
       Gtk::TreeModel::Row row = *iter;
-      if(row[m_model_fields->m_columns.m_col_is_group])
+      if(row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP)
       {
         //Add a group under this group:
         parent = iter;
@@ -485,38 +580,91 @@ void Dialog_Layout_Details::on_button_field_add_group()
     }
   }
 
+  return parent;
+}
+
+void Dialog_Layout_Details::on_button_field_add_group()
+{
+  Gtk::TreeModel::iterator parent = get_selected_group_parent();
+
   Gtk::TreeModel::iterator iterNewGroup;
   if(!parent)
   {
     //Add it at the top-level, because nothing was selected:
-    iterNewGroup = m_model_fields->append();
+    iterNewGroup = m_model_items->append();
   }
   else
   {
-    iterNewGroup = m_model_fields->append(parent->children());
+    iterNewGroup = m_model_items->append(parent->children());
   }
 
   if(iterNewGroup)
   {
     Gtk::TreeModel::Row row = *iterNewGroup;
-    row[m_model_fields->m_columns.m_col_is_group] = true;
-    row[m_model_fields->m_columns.m_col_name] = "Untitled group";
-    row[m_model_fields->m_columns.m_col_columns_count] = 1;
+    row[m_model_items->m_columns.m_col_type] = TreeStore_Layout::TYPE_GROUP;
+    row[m_model_items->m_columns.m_col_name] = "Untitled group";
+    row[m_model_items->m_columns.m_col_columns_count] = 1;
 
     //Scroll to, and select, the new row:
+    Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
     if(refTreeSelection)
       refTreeSelection->select(iterNewGroup);
 
     m_treeview_fields->scroll_to_row( Gtk::TreeModel::Path(iterNewGroup) );
-          
+
     m_modified = true;
   }
 }
 
 void Dialog_Layout_Details::on_button_field_edit()
 {
-  //TODO
+  //Get the selected item:
+  Glib::RefPtr<Gtk::TreeSelection> refTreeSelection = m_treeview_fields->get_selection();
+  if(refTreeSelection)
+  {
+    Gtk::TreeModel::iterator iter = refTreeSelection->get_selected();
+    if(iter)
+    {
+      //Do something different for each type of item:
+      //This is unpleasant, but so is this whole dialog.
+      //This whole dialog is just a temporary way to edit the layout before we have a visual DnD way.
+      Gtk::TreeModel::Row row = *iter;
+      switch( row[m_model_items->m_columns.m_col_type])
+      {
+        case TreeStore_Layout::TYPE_GROUP:
+        {
+          //TODO: Start editing the name.
+          break;
+        }
+        case TreeStore_Layout::TYPE_FIELD:
+        {
+          Field field;
+          field.set_name( row[m_model_items->m_columns.m_col_name] ); //Start with this one selected.
+          bool test = offer_field_list(field);
+          if(test)
+          {
+            row[m_model_items->m_columns.m_col_name] = field.get_name();
+          }
+          
+          break;
+        }
+        case TreeStore_Layout::TYPE_PORTAL:
+        {
+          Relationship relationship;
+          relationship.set_name( row[m_model_items->m_columns.m_col_relationship] ); //Start with this one selected.
+          bool test = offer_relationship_list(relationship);
+          if(test)
+          {
+            row[m_model_items->m_columns.m_col_relationship] = relationship.get_name();
+          }
+          
+          break;
+        }
+      }
+    }
+  }
 }
+
 void Dialog_Layout_Details::save_to_document()
 {
   Dialog_Layout::save_to_document();
@@ -531,7 +679,7 @@ void Dialog_Layout_Details::save_to_document()
 
     //Fill the sequences:
     //(This model is not sorted - we need to set the sequence numbers based on the order).
-    m_model_fields->fill_sequences();
+    m_model_items->fill_sequences();
 
     //Get the groups and their fields:
     Document_Glom::type_mapLayoutGroupSequence mapGroups;
@@ -539,19 +687,20 @@ void Dialog_Layout_Details::save_to_document()
 
     //Add the layout items:
     //guint field_sequence = 1; //0 means no sequence
-    for(Gtk::TreeModel::iterator iterFields = m_model_fields->children().begin(); iterFields != m_model_fields->children().end(); ++iterFields)
+    for(Gtk::TreeModel::iterator iterFields = m_model_items->children().begin(); iterFields != m_model_items->children().end(); ++iterFields)
     {
       Gtk::TreeModel::Row row = *iterFields;
-      if(row[m_model_fields->m_columns.m_col_is_group]) //There may be top-level groups, but no top-level fields, because the fields must be in a group (so that they are in columns)
+      if(row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP) //There may be top-level groups, but no top-level fields, because the fields must be in a group (so that they are in columns)
       {
         LayoutGroup group;
+        group.m_sequence = group_sequence;
         fill_group(iterFields, group);
 
         mapGroups[group_sequence] = group;
         ++group_sequence;
       }
     }
-   
+
     if(m_document)
     {
       m_document->set_data_layout_groups(m_layout_name, m_table_name, mapGroups);
@@ -567,6 +716,8 @@ void Dialog_Layout_Details::on_treeview_fields_selection_changed()
 
 void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const Gtk::TreeModel::iterator& iter)
 {
+  //TODO: If we ever use this a real layout tree, then let's add icons for each type.
+  
   //Set the view's cell properties depending on the model's data:
   Gtk::CellRendererText* renderer_text = dynamic_cast<Gtk::CellRendererText*>(renderer);
   if(renderer_text)
@@ -575,19 +726,27 @@ void Dialog_Layout_Details::on_cell_data_name(Gtk::CellRenderer* renderer, const
     {
       Gtk::TreeModel::Row row = *iter;
 
-      const bool is_group = row[m_model_fields->m_columns.m_col_is_group];
-      
+      const bool is_group = row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP;
+      const bool is_relationship = row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_PORTAL;
+
       //Use the markup property instead of the text property, so that we can give the text some style:
       Glib::ustring markup;
       if(is_group)
       {
         //Make group names bold:
-        markup = "<b>" + row[m_model_fields->m_columns.m_col_name] + "</b>";
+        markup = "<b>" + row[m_model_items->m_columns.m_col_name] + "</b>";
+      }
+      else if(is_relationship)
+      {
+        markup = gettext("Related: ") + row[m_model_items->m_columns.m_col_relationship];
       }
       else
-        markup = row[m_model_fields->m_columns.m_col_name];
+        markup = row[m_model_items->m_columns.m_col_name];
 
       renderer_text->property_markup() = markup;
+
+      renderer_text->property_editable() = row[m_model_items->m_columns.m_col_type] != TreeStore_Layout::TYPE_PORTAL; //Portals can only be edited via the Edit button.
+ 
     }
   }
 }
@@ -602,8 +761,8 @@ void Dialog_Layout_Details::on_cell_data_title(Gtk::CellRenderer* renderer, cons
     {
       Gtk::TreeModel::Row row = *iter;
 
-      renderer_text->property_text() = row[m_model_fields->m_columns.m_col_title];
-      renderer_text->property_editable() = row[m_model_fields->m_columns.m_col_is_group]; //Only groups have titles that can be edited.
+      renderer_text->property_text() = row[m_model_items->m_columns.m_col_title];
+      renderer_text->property_editable() = row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP; //Only groups have titles that can be edited.
     }
   }
 }
@@ -618,7 +777,7 @@ void Dialog_Layout_Details::on_cell_data_columns_count(Gtk::CellRenderer* render
     {
       Gtk::TreeModel::Row row = *iter;
 
-      const bool is_group = row[m_model_fields->m_columns.m_col_is_group]; //Only groups have column_count that can be edited.
+      const bool is_group = row[m_model_items->m_columns.m_col_type] == TreeStore_Layout::TYPE_GROUP; //Only groups have column_count that can be edited.
 
       //Get a text representation of the number:
       Glib::ustring text;
@@ -626,7 +785,7 @@ void Dialog_Layout_Details::on_cell_data_columns_count(Gtk::CellRenderer* render
       {
         std::stringstream the_stream;
         //the_stream.imbue(std::locale::classic());
-        guint number = row[m_model_fields->m_columns.m_col_columns_count];
+        guint number = row[m_model_items->m_columns.m_col_columns_count];
         the_stream << number;
         text = the_stream.str();
       }
@@ -643,26 +802,31 @@ void Dialog_Layout_Details::on_cell_data_columns_count(Gtk::CellRenderer* render
 
 void Dialog_Layout_Details::on_treeview_cell_edited_text(const Glib::ustring& path_string, const Glib::ustring& new_text, const Gtk::TreeModelColumn<Glib::ustring>& model_column)
 {
- Gtk::TreePath path(path_string);
-
-  //Get the row from the path:
-  Gtk::TreeModel::iterator iter = m_model_fields->get_iter(path);
-  if(iter)
+  if(!path_string.empty())
   {
-    //Store the user's new text in the model:
-    Gtk::TreeRow row = *iter;
-    row[model_column] = new_text;
+    Gtk::TreePath path(path_string);
+
+    //Get the row from the path:
+    Gtk::TreeModel::iterator iter = m_model_items->get_iter(path);
+    if(iter)
+    {
+      //Store the user's new text in the model:
+      Gtk::TreeRow row = *iter;
+      row[model_column] = new_text;
+    }
   }
 }
-  
+
 void Dialog_Layout_Details::on_treeview_cell_edited_numeric(const Glib::ustring& path_string, const Glib::ustring& new_text, const Gtk::TreeModelColumn<guint>& model_column)
 {
   //This is used on numerical model columns:
+  if(path_string.empty())
+    return;
 
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
-  Gtk::TreeModel::iterator iter = m_model_fields->get_iter(path);
+  Gtk::TreeModel::iterator iter = m_model_items->get_iter(path);
   if(iter)
   {
     //std::istringstream astream(new_text); //Put it in a stream.
