@@ -415,7 +415,7 @@ bool AddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, boo
            g_warning("AddDel::select_item:TreeViewColumn not found.");
       }
       else
-           g_warning("AddDel::select_item:TreeViewColumn index not found.");
+           g_warning("AddDel::select_item:TreeViewColumn index not found. column=%d", column);
     }
 
     bResult = true;
@@ -863,7 +863,7 @@ Glib::ustring AddDel::get_column_field(guint column_index) const
 {
   Glib::ustring result;
   if(column_index < m_ColumnTypes.size())
-    result = m_ColumnTypes[column_index].m_name;
+    result = m_ColumnTypes[column_index].m_id;
 
   return result;
 }
@@ -1042,6 +1042,9 @@ Glib::ustring AddDel::treeview_get_key(const Gtk::TreeModel::iterator& row)
 
 void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index)
 {
+  if(path_string.empty())
+    return;
+    
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
@@ -1114,6 +1117,9 @@ void AddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, int
 
 void AddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index)
 {
+  if(path_string.empty())
+    return;
+  
   Gtk::TreePath path(path_string);
 
   //Get the row from the path:
